﻿using System;

namespace CySoft.Collections.Geometry
{
	/// <summary>
	/// Class for tree with Unweighted Squared Euclidean distancing.
	/// </summary>
	/// <typeparam name="T">Value type</typeparam>
	public class SqrEuclidKDTree<T> : KDTree<T>
	{
		public SqrEuclidKDTree(int dimensions)
			: base(dimensions, null)
		{
		}

		public SqrEuclidKDTree(int dimensions, int sizeLimit)
			: base(dimensions, sizeLimit)
		{
		}

		protected override double PointDist(double[] p1, double[] p2)
		{
			double d = 0;

			for (int i = 0; i < p1.Length; i++) {
				double diff = (p1[i] - p2[i]);
				if (!Double.IsNaN(diff)) {
					d += diff * diff;
				}
			}

			return d;
		}

		protected override double PointRegionDist(double[] point, double[] min, double[] max)
		{
			double d = 0;

			for (int i = 0; i < point.Length; i++) {
				double diff = 0;
				if (point[i] > max[i]) {
					diff = (point[i] - max[i]);
				} else if (point[i] < min[i]) {
					diff = (point[i] - min[i]);
				}

				if (!Double.IsNaN(diff)) {
					d += diff * diff;
				}
			}

			return d;
		}
	}

}
