﻿using System.Diagnostics;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CySoft.Collections.Geometry;
using System;

namespace Tests.Benchmark
{
	class Program
	{
		const int N_Points = 100000, N_Neighors = 5000, N_Neighors_Times = 50;
		const int TotalingRepetitions = 20, AveragingRepetitions = 10;

		private static readonly Random rand = new Random();
		private static readonly Stopwatch stopwatch = new Stopwatch();

		static void Main(string[] args)
		{
			long totAddPtsTime = 0;
			long txtFndNgbrsTime = 0;

			for (int i = 0; i < AveragingRepetitions; i++) {
				Console.Write("Add {0} points {1} times", N_Points, TotalingRepetitions);
				long addPtsTime = MesasureAddingPoints();
				totAddPtsTime += addPtsTime;
				Console.WriteLine("   ==> Total = {0} ms ", addPtsTime);

				Console.Write("Get {0} neighbors from {1} points {2}×{3} times", N_Neighors, N_Points, TotalingRepetitions, N_Neighors_Times);
				long fndNgbrsTime = MeasureFindingNeighbors();
				txtFndNgbrsTime += fndNgbrsTime;
				Console.WriteLine("   ==> Total = {0} ms ", fndNgbrsTime);
			}
			Console.WriteLine();
			Console.WriteLine("AVERAGED times from {0} measurements", AveragingRepetitions);
			Console.WriteLine("Add points = {0} ms      Get neigbors = {1} ms", totAddPtsTime / AveragingRepetitions, txtFndNgbrsTime / AveragingRepetitions);

			Console.WriteLine();
			Console.Write("Press key to exit!");
			Console.ReadKey();
		}

		private static long MeasureFindingNeighbors()
		{
			var tree = AddPoints(N_Points);
			var dist = tree.NearestNeighbors(1000, rand.NextDouble(), rand.NextDouble(), rand.NextDouble()).Max(n => n.Distance); // Warmup
			long totTime = 0;
			for (int i = 0; i < TotalingRepetitions; i++) {
				stopwatch.Reset();
				stopwatch.Start();
				for (int k = 0; k < N_Neighors_Times; k++) {
					dist = tree.NearestNeighbors(N_Neighors, rand.NextDouble(), rand.NextDouble(), rand.NextDouble()).Max(n => n.Distance);
				}
				stopwatch.Stop();
				totTime += stopwatch.ElapsedMilliseconds;
				// Console.WriteLine("{0} ms ", stopwatch.ElapsedMilliseconds);
			}
			return totTime;
		}

		private static long MesasureAddingPoints()
		{
			AddPoints(100); // Warmup
			long totTime = 0;
			for (int i = 0; i < TotalingRepetitions; i++) {
				stopwatch.Reset();
				stopwatch.Start();
				AddPoints(N_Points);
				stopwatch.Stop();
				totTime += stopwatch.ElapsedMilliseconds;
				// Console.WriteLine("{0} ms ", stopwatch.ElapsedMilliseconds);
			}
			return totTime;
		}

		private static KDTree<int> AddPoints(int n)
		{
			var tree = new KDTree<int>(3);
			for (int i = 0; i < n; i++) {
				tree.Add(i, rand.NextDouble(), rand.NextDouble(), rand.NextDouble());
			}
			return tree;
		}

	}
}
