﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using CySoft.Collections.Geometry;

namespace Tests.OnConsole
{
	static class VisualKDTreeTests
	{
		public static void PrintEulidianDistance()
		{
			Console.WriteLine("Eulidian distance");
			PrintMatrix(SquareEuclideanDistance.Instance, 61);
		}

		public static void PrintManhattanDistance()
		{
			Console.WriteLine("Manhattan distance");
			PrintMatrix(ManhattanDistance.Instance, 61);
		}

		public static void PrintChebyshevDistance()
		{
			Console.WriteLine("Chebyshev distance");
			PrintMatrix(ChebyshevDistance.Instance, 49);
		}

		private static void PrintMatrix(DistanceFunction strategy, int pointCount)
		{
			const int N = 11;

			var kdTree = new KDTree<Point>(2);

			// Fill Tree
			for (int x = 0; x < N; x++) {
				for (int y = 0; y < N; y++) {
					kdTree.Add(new Point(x, y), x, y);
				}
			}

			// Get Neighbors
			var neighbors = kdTree.NearestNeighbors(strategy, pointCount, N / 2, N / 2);

			// Print result
			var dict = new HashSet<Point>(neighbors.Select(n => n.Value));
			for (int x = 0; x < N; x++) {
				for (int y = 0; y < N; y++) {
					if (dict.Contains(new Point(x, y))) {
						Console.Write("O ");
					} else {
						Console.Write("· ");
					}
				}
				Console.WriteLine();
			}
		}
	}
}
